import java.io.*;
import java.net.*;


/**
* Classed used buy the server to handle connections to clients
* @author Vetle Valebjrg
* @version 1.0
* @see Mofn
*/
class Client{
		private	Socket sock;
		private ObjectOutputStream outs;
		private ObjectInputStream ins;
		private ServerSocket server;
		private boolean running;
		public int start;
		public int stop;
		
		/**
		* Default constructor
		*/
		public Client(){
		}
		
		/**
		* Starts a client connection with portnr, startpos and stoppos in motiflist
		* @param portNr startnr.
		* @param c Offset for client to portnr.
		* @param start start position in motif group to handle.
		* @param stop End position in motif group to handle.
		*/
		public Client(int portNr, int c, int start, int stop){
			try{
				server = new ServerSocket(portNr +c,0,InetAddress.getLocalHost());
				
				sock = server.accept();
				
				outs = new ObjectOutputStream(sock.getOutputStream());
				ins = new ObjectInputStream(sock.getInputStream());
				
				this.start = start;
				this.stop = stop;
				
				running = true;
			
			}catch(IOException e){
				System.out.println(e.getMessage());
			}
		}
		
		/**
		* Returns the objectoutputstream for the client handler
		* @return Link to stream used.
		*/
		public ObjectOutputStream getOOS(){
			return outs;
		}
		
		/**
		* Returns the objectinputstream for the client handler
		* @return Link to stream used.
		*/
		public ObjectInputStream getOIS(){
			return ins;
		}
		
		/**
		* Returns the connected clients status
		* @return Returns true if client is active
		*/
		public boolean isRunning(){
			return running;
		}
		
		/**
		* Stops a client if needed
		* @throws IOException
		*/
		public void stopClient() throws Exception{
			running = false;
			try{
				sock.close();
			}catch(Exception e){
				e.printStackTrace();
			}
		}		
		
		/**
		* Returns the connections IP address
		* @return IP address used by the socket.
		*/
		public InetAddress getIP(){
			return sock.getInetAddress();
		}
	}