import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;

 /**
 * This class holds the list of Single Motifs, all motif variants
 * @author Vetle Valebjrg
 * @version 1.0
 * @see SingleMotif, RealCompositeMotif, FictiveCompositeMotif, MotifVariant
 */
public class CompositeMotif implements Cloneable{
	
	protected HashMap motVarHolder;
	protected ArrayList singleMotifHolder;
	
	private String cName = "";
	
	/**
	* Default constructor
	*/
	public CompositeMotif(){
		motVarHolder = new HashMap();
		singleMotifHolder = new ArrayList();
	}
	
	/**
	* Constructor used during cloning
	* @param cm Original composite motif to copy.
	*/
	public CompositeMotif(CompositeMotif cm){
		motVarHolder = (HashMap)cm.motVarHolder.clone();
		singleMotifHolder = (ArrayList)cm.singleMotifHolder.clone();
	}
	
	/**
	* Sets a new name
	* @param str The new name.
	*/		
	public void setName(String str){
		cName = str;
	}

	/**
	* Returns the Composite Motifs name
	* @return Name
	*/
	public String getName(){
		return cName;
	}
	
	/**
	* Makes keys used to identify motif variants Nr-Nr
	* @param mt Part of value to make a key for the motif variant m of n
	* @param nt Part of value to make a key for the motif variant m of n
	*/
	protected String makeKey(int mt, int nt){
		return Integer.toString(mt) +"-"+ Integer.toString(nt);
	}
	
	/**
	* Handles calculations of motif variants
	* @param nt Current n calculated
	* @param sm Single motif added during expansion
	* @param expType Type of expansion (expand/prune) - used for debug
	* @return Higest m-value calculated (used by pruning).
	*/
	protected int calc(int nt, SingleMotif sm, String expType){
		
		int lowl = nt-Mofn.MAX_M;
		if(lowl <0){
			lowl=0;
		}
		
		int highl = Mofn.MAX_M;
		if(highl>nt){
			highl=nt;
		}

		//Iterating thorugh each column for current N
		int t=0;
		for(t=lowl;t<=highl;t++){
			//Mofn.add_output("Action: " + expType);
				
			//Creating MV for gray area above N/N combinations
			if(t==nt && nt!=0){
				MotifVariant outOfBoundsMV = new MotifVariant();
				outOfBoundsMV.makeDummy();
				motVarHolder.put(makeKey(nt+1,nt),outOfBoundsMV);
				
//				Mofn.add_output("Expand: id:" + outOfBoundsMV.getId() + ": " + (nt + 1) + " - " + nt);
				//Mofn.add_output("Sign: " + outOfBoundsMV.getSign());
				//Mofn.add_output("Supp: " + outOfBoundsMV.getBitSet().toString());	
			}
			
			if(t==0 && nt>0){
				//STARTUP
				MotifVariant firstMV = new MotifVariant();
				firstMV.makeFirst();
				motVarHolder.put(makeKey(t,nt),firstMV);
				
//				Mofn.add_output("Expand: id: " + firstMV.getId() + ": " + t + " - " + nt);
				//Mofn.add_output("Sign: " + firstMV.getSign());
				//Mofn.add_output("Supp: " + firstMV.getBitSet().toString());	
			
			}else{
				//EXPAND
				MotifVariant newMV = new MotifVariant();
				
				MotifVariant prevMV =(MotifVariant)motVarHolder.get(makeKey(t-1,nt-1));
				MotifVariant dummyMV =(MotifVariant)motVarHolder.get(makeKey(t,nt-1));
				
//				Mofn.add_output("Expand: id: " + newMV.getId() +":"+prevMV.getId()+":"+dummyMV.getId() + ": " + t + " - " + nt);
				
				newMV = newMV.calc_mn(prevMV,dummyMV,sm);
				
				//System.out.println("RES: " + newMV.getBitSet().toString());
				motVarHolder.put(makeKey(t,nt),newMV);
					
			}//else
		}//for
		return t-1;
	}//calc
	
	
	/** 
	* This method returns the list of motifs held in object
	* @return List of motifs participating in the module.
	*/
	public ArrayList getSingelMotifs(){
		return singleMotifHolder;
	}
	
	/**
	* Method added to override object clone
	* @return A copy of the module.
	* @see Object
	*/
	public Object clone(){
		try {
	  		return super.clone(); 
    	}catch (CloneNotSupportedException e) { // Dire trouble!!!
         	throw new InternalError("But we are Cloneable!!!");
      	}
	}
	
}
	
	