import java.util.ArrayList;
import java.util.BitSet;

/**
 * This class is used for pruning, which narrow the DFS enumeration
 * @author Vetle Valebjrg
 * @version 1.0
 * @see CompositeMofif, RealCompositeMotif, SingleMotif
 */
public class FictiveCompositeMotif extends CompositeMotif implements Cloneable{
	
	
	/**
	* Default constructor used by subclass Motifs
	* @see Motif
	*/
	public FictiveCompositeMotif(){
	}
	
	/**
	* Constructor used during cloning of composite elements
	*/
	public FictiveCompositeMotif(CompositeMotif cm){
		super(cm);
	}
	
	/**
	* Method used to prune and narrow the enumeration during DFS
	* @param n Position in motiflist
	* @param lsm_sign Last single motif significance.
	* @param left Distance left before MAX_N.
	* @return Best possible significance found during prune.
	*/
	public double prune(int n,double lsm_sign, int left){
		
		double bestres = lsm_sign;
		
		SingleMotif fsm = new SingleMotif(Mofn.nr_of_seq, Mofn.nr_of_seq,lsm_sign);	
		singleMotifHolder.add(fsm);
		
		int nt = singleMotifHolder.size();
		int t = calc(nt,fsm,"pruning");		

		while(n<Mofn.motifList.size()-1 && nt<Mofn.MAX_N && left>0){
						
			n++;
			left--;
			//PRUNE, CHECK TO SEE IF SEARCH CAN BE STOPPED
			FictiveCompositeMotif fcm = new FictiveCompositeMotif((CompositeMotif)this.clone());
			bestres = fcm.prune(n,lsm_sign,left);
			//if(nt==Mofn.Max_N){
			//	bestres = fcm.getSigngetSign();
			//}
			//if(singleMotifHolder.size()<Mofn.MAX_N){
			//	bestres = this.prune(n,lsm_sign);
				
			//}//if
			
		}//while
		//HashMap hm = fcm.getMotVarHolder();
		if(nt==Mofn.MAX_N){
			MotifVariant chkMV = (MotifVariant)motVarHolder.get(makeKey(Mofn.MAX_M,Mofn.MAX_N));
			bestres = chkMV.getSign();
		}
		return bestres;
	}//prune

	
	
	/**
	* Method added to override object clone, which then enables cloning of CMs
	* @return A copy of this module
	* @see Object
	*/
	public Object clone(){
		try {
	  		return super.clone(); 
    	}catch (Exception e) { // Dire trouble!!!
         	throw new InternalError("But we are Cloneable!!!");
      	}
	}
}
	
