import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.BitSet;
import java.io.*;

import java.math.*;
import java.util.HashMap;

/**
 * This class handles the input from files and test-data.
 * @author Vetle Valebjrg
 * @version 2.0
 */
public class InputReader{
	
	private String inFile = "data//input.txt";
	private int highestSeq = 0;
	HashMap freq = new HashMap();
	
	/**
	* Default constructor
	*/
	public InputReader(){
		setDefaultSign();
	}
	
	/**
	* Constructor that takes a filename (String) as parameter)
	* @param str Filename as string
	*/
	public InputReader(String mFile){
			inFile = mFile;
			setDefaultSign();
	}	
	
	private void setDefaultSign(){		
		freq.put("A", new Double(0.25));
		freq.put("C", new Double(0.25));
		freq.put("G", new Double(0.25));
		freq.put("T", new Double(0.25));
		freq.put("a", new Double(0.25));
		freq.put("c", new Double(0.25));
		freq.put("g", new Double(0.25));
		freq.put("t", new Double(0.25));
		freq.put(".", new Double(1.00));
	}
	
	
	/*
	public void makeTestMotifs(int number){
		Vector motpatterns = createRandomSequences(number,7);
		for(int i=0;i<number;i++){
			SingleMotif testMotif = new SingleMotif(20,20,0.90,motpatterns.get(i*2+1),new int[20],new BitSet(20));	
			testMotif.setName(Integer.toString(i));
			motifList.add();
		}
	}	
	*/
	
	/**
	* Creates a set of random sequence patterns
	* @param nrseq Number of random sequences
	* @param lengthseq Length of random sequences
	* @return Sequence list (name, patterns)
	*/
	public static Vector createRandomSequences(int nrseq, int lengthseq){

		int[] acgt = {65,67,71,84};
		Vector seqs = new Vector();
		
		for(int i=0;i<nrseq;i++){
			String seq = "";
			seqs.add(new Integer(i));
					
			for(int l=0;l<lengthseq;l++){
	
				seq=seq + (char)acgt[((int)(Math.random()*4))];
			}
			
			seqs.add(seq);
		}
		return seqs;
	}
	
	/**
	* Calculates the frequencies for A,C,G and T in the
	* sequences provied (name, sequence).
	* @param seq List of sequences (name, pattern)
	*/
	public void setSignBasedOnBackground(Vector seqs){
		double totA = 0.0;
		double totC = 0.0;
		double totG = 0.0;
		double totT = 0.0;
		 
		for(int i=1;i<seqs.size();i+=2){
			String backstr = (String)seqs.get(i);
			backstr = backstr.toUpperCase();
			int sumA = 0;
			int sumC = 0;
			int sumG = 0;
			int sumT = 0;
		
			for(int c=0;c<backstr.length();c++){

				if(backstr.substring(c,c+1).equals("A")){
					sumA++;
				}else if(backstr.substring(c,c+1).equals("C")){
					sumC++;
				}else if(backstr.substring(c,c+1).equals("G")){
					sumG++;
				}else{
					sumT++;
				}		
			}
			totA = (double)sumA/backstr.length();
			totC = (double)sumC/backstr.length();
			totG = (double)sumG/backstr.length();
			totT = (double)sumT/backstr.length();
		}
		
		freq.put("A", new Double(totA));
		freq.put("C", new Double(totC));
		freq.put("G", new Double(totG));
		freq.put("T", new Double(totT));
		freq.put("a", new Double(totA));
		freq.put("c", new Double(totC));
		freq.put("g", new Double(totG));
		freq.put("t", new Double(totT));
		freq.put(".", new Double(1.00));
		
	}
	
	/**
	* Returns number of sequences found in input-files
	* @return Nr of sequences found
	*/
	public int getNrSeq(){
		ArrayList trash = readMotifs(10);
		//System.out.println("Seqnr: " + (highestSeq+1));
		return highestSeq;
		
	}
	
	/**
	* This method reads the data from the filename provied
	* @return Returns a list of single motifs objects read from file.
	* @exception IOException
	* @see IOException
	*/
	public ArrayList readMotifs(int nrofseq){
		ArrayList listOfMotifs = new ArrayList();
		
		try{
			
			FileReader myFIS = new FileReader(inFile);
			
			BufferedReader br = new BufferedReader(myFIS);
			String str;
						
			while((str = br.readLine())!=null){
				
				//System.out.println(str);

				String mName = "";
				double mSign=0.0;
				int mTotHit=0;
				int mAntSeq=0;
				int[] mSeqHitList = new int[0];
				BitSet mBitset = new BitSet(nrofseq);
						

				StringTokenizer stt;
				StringTokenizer sts;
						
				if(str.length()!=0 && !(str.startsWith("#"))){
					stt = new StringTokenizer(str,"	");
					
						int calc = stt.countTokens();
						//System.out.println("Normal sign"); ==4
						for(int i=0; i<=calc-(5-calc);i++){
							if(i == 0){
								mTotHit = Integer.parseInt(stt.nextToken());
								//System.out.println("Tot treff: " + stt.nextToken());
							}else if(i == 1){
								mAntSeq = Integer.parseInt(stt.nextToken());
								//System.out.println("Ant sekv: " + stt.nextToken());	
							}else if(i == 2){ //NOT RUN IF MANUAL stt.count==3
								mSign = Double.parseDouble(stt.nextToken());
								//System.out.println("Sign: " + mSign);
							}//if
							
						}//for
					
					String newStr = stt.nextToken();
					sts = new StringTokenizer(newStr," ");
					mName = sts.nextToken();
					//System.out.println("Mnster: " + mName);
					
					//MANUAL CALC OF SIGN (IF NOT IN INPUT)
					if(calc==3){
						//System.out.println(mName);
						mSign = calc_sign(mName);
						//System.out.println(mSign);
					}
					
					mSeqHitList = new int[sts.countTokens()];
					int teller = 0;
					//System.out.println("Lengde: " + mSeqHitList.length);
					
					while(sts.hasMoreTokens()){
						int listSeq = Integer.parseInt(sts.nextToken());
						int listPos = Integer.parseInt(sts.nextToken());
				
					
	
						if(listSeq>highestSeq){
							highestSeq = listSeq;
								/*	System.out.println(listSeq);
								try{
								Thread.sleep(1000);
							}catch(Exception e){
								System.out.println("Svn avbrutt");
							}*/
						}
						
						mBitset.set(listSeq,true);
						
						mSeqHitList[teller] = listSeq;
						mSeqHitList[teller+1] = listPos;			
						teller+=2;
						//System.out.println(listInt);
					}
					//System.out.println("Len: "+ mSeqHitList.length);
					
					//SETT INN FRSTE I LISTA		
					if(listOfMotifs.size() ==0){
						//System.out.println(mSign); 
						SingleMotif newMotif = new SingleMotif(mTotHit, mAntSeq, mSign, mName, mSeqHitList,mBitset);
						newMotif.setName(mName);//"Motif");//-"+listOfMotifs.size());
						listOfMotifs.add(newMotif);
					}else{
					//ELLER SORTER ETTER SYNKENDE SIGNIFICANS
						
						SingleMotif newMotif = new SingleMotif(mTotHit, mAntSeq, mSign, mName, mSeqHitList,mBitset);
						newMotif.setName(mName);//"Motif");//-"+listOfMotifs.size());
						//int ms = 0;
						
						//ASC
						for(int i=0;i<listOfMotifs.size();i++){
							SingleMotif inListMotif = (SingleMotif)listOfMotifs.get(i);
							
							if(newMotif.getSign()<=inListMotif.getSign()){
								listOfMotifs.add(i,newMotif);
								newMotif=null;
								break;
							}
						}
						if(newMotif!=null){
							listOfMotifs.add(newMotif);
							newMotif=null;
						};//(ms<= listOfMotifs.size() && 
						
						/*
						//DESC
						do{
							SingleMotif inListMotif = (SingleMotif)listOfMotifs.get(ms);
							
							if (ms>=listOfMotifs.size()-1){
								listOfMotifs.add(newMotif);
								newMotif=null;
							}else if(newMotif.getSign()>=inListMotif.getSign()){
								listOfMotifs.add(ms,newMotif);
								newMotif=null;
							}
							ms++;	
						}while (newMotif!=null);//(ms<= listOfMotifs.size() && 
						*/
					}//if
					
				}//if
			}//while
			
			myFIS.close();
			
		}catch(Exception e){
			//ERROR READING INPUT
			System.out.println(e);
		}
		
		return listOfMotifs;
	}//function
	
	/**
	* Returns the filename used.
	* @return filname
	*/
	public String getFile(){
		return inFile;
	}
	
	/**
	* This method takes the 2-logarithm of a double
	* @param num as double
	* @return Answer as 2-log of num
	*/
	private double log2(double num){
		return Math.log(num);
	}
	
	/**
	* This method calculates the significance for a pattern
	* based on the frequency of letters and sequence length. (log removed since previous project.)
	* @param pattern String representation of pattern
	* @return The significance found.
	*/
	private double calc_sign(String pattern){
		
		int datalen = 1000;
		if(Mofn.sequenceList.size()>0){
			datalen = Mofn.getAvgSeqLen();
		}
		

		
		double sign = 1.0;
    	double logsign = 0.0;
    	double ambsign = 1.0;
    	int length = 0;
    	boolean inamb = false;

	    for(int i=0;i<pattern.length()-1;i++){
	        String keystr = pattern.substring(i,i+1);
	        //System.out.println(keystr);
	        //System.out.println(((Double)freq.get(keystr)).doubleValue());
	        
	        //A..G.AAAA
	        if(!inamb){
	            if(freq.containsKey(keystr)){
	                logsign += log2(((Double)freq.get(keystr)).doubleValue());
	                sign = sign * ((Double)freq.get(keystr)).doubleValue();
	                length += 1;
	
	            }else if(keystr.equals("[")){
	                ambsign = 0.0;
	                inamb = true;
	                continue;
	            }
	        }else if(freq.containsKey(keystr)){
	            ambsign += ((Double)freq.get(keystr)).doubleValue();
	
	        }else if(keystr.equals("]")){
	            sign *= ambsign;
	            logsign += log2(ambsign);
	            length += 1;
	            inamb = false;
	
	        }else{
	            System.out.println("unknown symbol in pattern");
	        }
		}
		
	    if(length>=datalen){
	        //return logsign;
	        return sign;
	    //}else if(logsign < -40){
	        //System.out.println("sign1");
	        //return (logsign + log2(datalen - length));
	      //  return (sign + datalen - length);
	        
	    }
	    else{
	    	//System.out.println("Sign2");
	        //return (log2(1.0 - Math.pow(1.0 - sign, datalen - length)));
	        return 1.0 - Math.pow(1.0 - sign, datalen - length);
	        
	    }//if
	}//for
	
	/**
	* Reads next number used with output files
	* @param nrFile Filename containing the next output nr
	* @return The next number that should be used.
	*/
	public String readNextOutputFileNr(String nrFile){
		String retStr = "";
		
		try{
			
			FileReader myFR = new FileReader(nrFile);
			
			BufferedReader br = new BufferedReader(myFR);
						
			retStr = br.readLine();
			
			myFR.close();
			
		}catch(Exception e){
			//ERROR READING INPUT
			System.out.println(e);
		}
		
		return retStr.trim();
	}
		
	/**
	* Reads all PWM matrices found in file provided
	* @param filename Filename
	* @return Motiflist with PWM motifs
	*/
	public ArrayList readMatrix(String filename){
		ArrayList listOfMotifs = new ArrayList();
		
		try{
			
			FileReader myFR = new FileReader(filename);
			BufferedReader br = new BufferedReader(myFR);
			
			String mName = "";
			int cols = 0;
			int rows = 0;
			int countRows = 0;
			Vector matrix = new Vector();
			
			boolean moretoread = true;
			
			String str;
			while(moretoread){
				str = br.readLine();
		
				if(str==null){
					str =">avslutt";
					moretoread = false;
				}
							
				if(!str.startsWith("#")){
					
					if(str.startsWith(">") && countRows==0){
						mName = str.substring(1);
					}else if(!str.startsWith(">")){
						
						//System.out.println("str: " + str);
							
						StringTokenizer stt = stt = new StringTokenizer(str," ");
						//System.out.println(str);
						countRows++;
						
						cols = stt.countTokens();
						
						while(stt.hasMoreTokens()){
							
							//System.out.println("-" + sttSub.nextToken()+"-");
							matrix.add(new Double(stt.nextToken()));
						}
					}			
				
					if(str.startsWith(">") && countRows!=0){
						rows = countRows;
						
						double[][] mList = new double[rows][cols];
						double[] rowsum = new double[rows];
						//System.out.println(">" + mName);
						for(int r=0;r<rows;r++){
							for(int c=0;c<cols;c++){
								double value = ((Double)matrix.elementAt(r*4+c)).doubleValue();
								//System.out.print(value + " ");
								mList[r][c] = value;
								rowsum[r]+=value;
							}
							//System.out.println(rowsum[r]);
						}
						
						SingleMotif sm = new SingleMotif(0,0,0.0,mName,new int[Mofn.nr_of_seq+1],new BitSet());
						sm.setMatrix(mList,rows,cols,rowsum);
						sm.setName(mName);
						listOfMotifs.add(sm);
						rows = 0;
						cols = 0;
						countRows=0;
						matrix = new Vector();
						mName = str.substring(1);

					}//if
				}//#
			}//while
		
			myFR.close();
			
		}catch(Exception e){
			e.printStackTrace();
		}//try
		
		return listOfMotifs;
	}//func
						
						
	/**
	* Reads all sequences found in file provided
	* @param filename Sequence filename
	* @return Sequences with alternating (name, pattern)
	*/		
	public Vector readSequences(String filename){
		
		Vector sequences = new Vector();
		
		try{
			
			FileReader myFR = new FileReader(filename);
			BufferedReader br = new BufferedReader(myFR);
			String str;
						
			while((str = br.readLine())!=null){

				String mName = "";
				String sequence = "";
				
				StringTokenizer stt;
						
				if(str.length()!=0 && !(str.startsWith("#"))){
					stt = new StringTokenizer(str,"	");
					
					if(str.startsWith(">")){
						mName=stt.nextToken();
						
						mName = mName.substring(1);
						sequences.add(mName);
					}else{
						sequences.add(stt.nextToken());
						
					}//else
				}//if
			}//while
			
			myFR.close();
			
		}catch(Exception e){
			e.printStackTrace();
		}//try
		highestSeq = sequences.size()/2;
		
		return sequences;
	}//func
	
		
	
}//class