//import java.io.*;
//import java.awt.*;
//import java.awt.event.*;

import java.util.Vector;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.StringTokenizer;
import java.util.HashMap;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Arrays;
import java.util.TreeMap;

import java.io.*;
import java.net.*;


/**
 * This class controlles the M of N application.
 * Starts standalone, server or client process.
 * Handles input (motifs, sequences), output (display, files)
 * @author Vetle Valebjrg
 * @version 1.0
 * @see CompositeMotif, RealCompositeMotif, FictiveCompositeMotif, SingleMotif, MotifVariant, Client, InputReader, OutputWriter
 */
public class Mofn{
	//CONSTANTS
	public static final String VERSION = "2.07.2006";
	private static final int MACHINE = 0;
	private static final int INPUT = 1;
	private static final int PVM = 1;
	private static final int MOT = 0;
	
	private static final int STANDALONE = 0;
	private static final int CLIENT = 1;
	private static final int SERVER = 2;
	
	private static final int PD_MOTIFS = 0;
	private static final int PD_PARETO = 1;
	private static final int PD_PATTERNS = 2;
	private static final int PD_SINGLEMOTIFS = 3;
	private static final int PD_COUNTS = 4;
	private static final int PD_WEIGHTS = 5;
	private static final int PD_BINOMIAL = 6;
	private static final int PD_EXP = 7;
	private static final int PD_FINAL = 8;
	private static int nroutput = 9; //change if list above increases
	
	private static BitSet display = new BitSet(nroutput);
	private static BitSet print = new BitSet(nroutput);
	
	//GLOBAL VARIABLES SET BY PROGRAM AND PARAMETERS
	private static Socket sock;
	private static ObjectInputStream ois;
	private static ObjectOutputStream oos;
	private static int portNr = 7777;
	private static int[] mode = {STANDALONE,PVM};
	private static int clients = 0;
	
	private static ArrayList outputstr = new ArrayList();
	private static ArrayList debugstr = new ArrayList();
	
	public static ArrayList motifList; //list of all motifs from input-file, sorted after significance
	public static Vector sequenceList;
	public static Vector backgroundList;
	
	public static boolean PRUNE = false;
	private static boolean ALIGN = false;
	
	public static int nr_of_seq;; //default number of sequences the input motif are retrieved from
	public static double[] pareto;  //list of significance for given support-index
	public static HashMap patterns = new HashMap();
	public static double threshold = 0.0;
	
	private static int startPos = 0; //startposition in the motifList
	private static int stopPos = 0;  //stopposition in the motifList
	private static int con_start = -1;
	private static int con_end = -1;
	
	//VALUES THAT CAN BE CHANGED
	public static int MAX_N = 2; //max combinations N = 3 finds m/3 (0<=m<=3)
	public static int MAX_M = 2; //number of m=goal, 0 = N 1 = N-1/N and so on.
	private static int percent = 30;
	public static String server_ip = "10.0.0.11";
	
	public static String sequenceFile = "data//TransCompel2//sequences//ap1-ets_seq.txt";
	public static String motifFile = "data//TransCompel2//mapped_matrices//ap1-ets_mat.txt";
	public static String backgroundFile = "";//data//Unique_5000_1000.fas";
	public static String outputFile = "output"; //default outputfile

	
	/**
	* Constructor with GUI if wanted
	*/
	public Mofn() {
	/*	addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
				System.exit(0);
			}
		});
		*/
	}
	
	/**
	* Parses the arguments provieded to Mofn and sets variables accordingly
	* @param args parameters used to control the program
	*/
	private static void extractParameters(String[] args){
		int machinemode = STANDALONE;
		int inputmode = PVM;
		print.set(0,nroutput);
		display.set(0,nroutput);
		int parameter = 0;
		
		while(parameter<args.length){
			if(args[parameter].equals("-mf")){
				parameter++;
				motifFile = args[parameter];
				
			}else if(args[parameter].equals("-type")){
				parameter++;
				if(args[parameter].equals("pwm")){
					inputmode=PVM;
				}else{
					inputmode=MOT;
				}
			}else if(args[parameter].equals("-sf")){
				parameter++;
				sequenceFile = args[parameter];
				
			}else if(args[parameter].equals("-of")){
				parameter++;
				if(args[parameter].indexOf(".")!=-1){
					outputFile = args[parameter].substring(0,args[parameter].indexOf("."));
				}else{
					outputFile = args[parameter];
				}
				
			}else if(args[parameter].equals("-bf")){
				parameter++;
				backgroundFile = args[parameter];
				
			}else if(args[parameter].equals("-m")){
				parameter++;
				MAX_M = Integer.parseInt(args[parameter]);
				
			}else if(args[parameter].equals("-n")){
				parameter++;
				MAX_N = Integer.parseInt(args[parameter]);
				
			}else if(args[parameter].equals("-mode")){
				parameter++;
				
				if(args[parameter].equals("SERVER")){
					machinemode = SERVER;
				}else if(args[parameter].equals("CLIENT")){
					machinemode = CLIENT;
				}else if(args[parameter].equals("STANDALONE")){
					machinemode = STANDALONE;
				}	
				
			}else if(args[parameter].equals("-cutoff")){
				parameter++;
				percent = Integer.parseInt(args[parameter]);
				
			}else if(args[parameter].equals("-ip")){
				parameter++;
				server_ip = args[parameter];
				
			}else if(args[parameter].equals("-port")){
				parameter++;
				portNr = Integer.parseInt(args[parameter]);
			
			}else if(args[parameter].equals("-print")){
				parameter++;
				for(int i=0;i<args[parameter].length();i++){
					if((args[parameter].substring(i,i+1)).equals("0")){
						print.clear(i);
					}
				}
			}else if(args[parameter].equals("-display")){
				parameter++;
				for(int i=0;i<args[parameter].length();i++){
					if((args[parameter].substring(i,i+1)).equals("0")){
						display.clear(i);
					}
				}
			}else if(args[parameter].equals("-clients")){
				parameter++;
				clients = Integer.parseInt(args[parameter]);
			}else if(args[parameter].equals("-start")){
				parameter++;
				con_start = Integer.parseInt(args[parameter]);
			}else if(args[parameter].equals("-end")){
				parameter++;
				con_end = Integer.parseInt(args[parameter]);
			}else if(args[parameter].equals("-prune")){
				PRUNE = true;
			}else if(args[parameter].equals("-align")){
				ALIGN = true;
			}
			
			parameter++;
			
		}//while parameters left
		
		mode[MACHINE] = machinemode;
		mode[INPUT] = inputmode; 
	}

	/**
	* This method is started at default by java
	* and analyses input parameters before it start run().
	* @param args list of parameters used to control the program.
	*/
	public static void main(String args[]) {		
							
		//DEFAULT RUN
		if(args.length!=1){
			
			//PARAMETER RUN  NORMAL/CLIENT/SERVER
			
			extractParameters(args);
			
			//READING PARAMETERS
			System.out.println("Reading in data...");
				
			//PVM
			if(mode[INPUT]==PVM){
				InputReader ir = new InputReader(sequenceFile);
				sequenceList = ir.readSequences(sequenceFile);
				
				if(backgroundFile.equals("")){
				}else{
					backgroundList = ir.readSequences(backgroundFile);
				//	ir.setSignBasedOnBackground(backgroundList);
				}
								
				motifList = ir.readMatrix(motifFile);
				
				nr_of_seq = sequenceList.size()/2;
				
				ArrayList scores = findMatrixScores(sequenceList);
				threshold = findMatrixThreshold(scores);
		
				findMatrixSupport(scores,threshold);
				if(backgroundFile.equals("")){
					findMatrixSign(threshold,null);
				}else{
					findMatrixSign(threshold,backgroundList);
				}
			}	
					
			//MOTIFS	
			if(mode[INPUT]==MOT){
				InputReader myReader = new InputReader(motifFile);
				sequenceList = myReader.readSequences(sequenceFile);
				
				nr_of_seq = myReader.getNrSeq();
				
				if(backgroundFile.equals("")){
					motifList = myReader.readMotifs(nr_of_seq);
				}else{
					backgroundList = myReader.readSequences(backgroundFile);
					if(ALIGN){
						motifList = myReader.readMotifs(nr_of_seq);
						findMotifSign(backgroundList);
					}else{
						myReader.setSignBasedOnBackground(backgroundList);
						motifList = myReader.readMotifs(nr_of_seq);
					}
				}
				
				
			}
			
			//removing 0-supp motifs
			for(int i=0;i<motifList.size();i++){
				SingleMotif sm = (SingleMotif)motifList.get(i);
				if(sm.getSupp()==0){
					motifList.remove(i);
					i--;
				}
			}
			
			//CLIENT/SERVER
			if(mode[MACHINE]==CLIENT){
				
				try{
					startSerializedClient(server_ip);
				}catch(Exception e){
					e.printStackTrace();
				
				}
			}else if(mode[MACHINE]==SERVER){
				
				try{
					startSerializedServer(clients);
				}catch(Exception e){
					e.printStackTrace();
				
				}
			}else{
				//STANDALONE
			}
			
			if(mode[MACHINE]!=CLIENT){
				
				System.out.println("");
				add_write_output("");
					
				//display_motifs();
				if(mode[MACHINE]!=SERVER){
					run();
				}
					
				//DO CHANGE TS FOR DISTANCE CONST)
				if(con_start!=-1 && con_end!=-1){
					applyDistanceConstraints(con_start,con_end);
				}
				
				System.out.println();
				add_write_output("");
				
				//PRINT
				display_settings(args);
				
				System.out.println();
				add_write_output("");
				
				display_pareto();
				System.out.println();
				add_write_output("");
				
				display_patterns();
				System.out.println();
				add_write_output("");
				
				display_mostCommonMotif();
				System.out.println();
				add_write_output("");
				
				int[] cSupp = countSupport();
				System.out.println();
				add_write_output("");
				
				weightedSupport(cSupp);
				System.out.println();
				add_write_output("");
				
				int bestNr = sign_binomial(cSupp);
				System.out.println();
				add_write_output("");
				
				sign_total(cSupp);
				System.out.println();
				add_write_output("");
	
				OutputWriter ow = new OutputWriter();
				ow.print_output(outputstr);				
				outputstr.clear();
	
				if(sequenceList!=null){
					//display_results(sequenceList,bestNr);
				}
				
				int cutpos = 0;
				if(motifFile.lastIndexOf("//")!=-1){
					cutpos = motifFile.lastIndexOf("//");
				}else{
					cutpos = motifFile.lastIndexOf("\\");
				}
						
				OutputWriter ow2 = new OutputWriter(motifFile.substring(cutpos+1,motifFile.length()-4)+"_" + MAX_M + MAX_N+"_");
				ow2.print_output(outputstr);				
						
				//ow2.print_output(debugstr);
			}
			
						

		}else if(args.length==1){
			//PRINT COMPLETE ARGUMENTLIST
			System.out.println("Version: " + VERSION);
			System.out.println("3 different running modes: STANDALONE, SERVER and CLIENT");
			System.out.println("Standard use: (-sf and -bf are optional but recommended)");
			System.out.println("STANDALONE: java Mofn -mf <motiffile> -type <mot/pvm> -sf <sequencefile> -bf <backgroundfile> -m <m> -n <n>");
			System.out.println("SERVER: java Mofn -mf <motiffile> -type <mot/pwm> -sf <sequencefile> -m <m> -n <n> -mode SERVER -clients <nr>");
			System.out.println("CLIENT: java Mofn -mf <motiffile> -type <mot/pwm> -sf <sequencefile> -bf <backgroundfile> -mode CLIENT -ip <xxx.xxx.xxx.xxx> -port 7777");
			System.out.println("");
			System.out.println("Parameterlist:");
			System.out.println("-mf <motiffile>");
			System.out.println("-sf <sequencefile>");
			System.out.println("-bf <backgroundfile>, used to alter significance for single motifs");
			System.out.println("-of <outputfile>, default='output.txt'");
			System.out.println("-m <m> m<=n, default=2");
			System.out.println("-n <n>, default=3");
			System.out.println("-type <mot/pvm>, default=pvm");
			System.out.println("-mode <STANDALONE/CLIENT/SERVER>, default=STANDALONE");
			System.out.println("-ip <xxx.xxx.xxx.xxx> Server IP address :");
			System.out.println("-port 7777");
			System.out.println("-cutoff <percent>, default=20");
			System.out.println("-print <111111111> 1 prints, 0 does not.");
			System.out.println("-display <111111111> 1 prints, 0 does not.");
			System.out.println("-start <distance start position (integer)>");
			System.out.println("-end <distance end position (integer)>");
			System.out.println("-prune, turns on pruning");
			System.out.println("-align, use alignment instead of frequencies for motifs (not PVM)");
			
			
		}
		
	}
			

	/**
	* Starts the DFS enumeration process. 
	*/
	private static void run(){
				
		//PRINT ARRAY SIZE OF INPUT
		//System.out.println("Size input data: "+ motifList.size());
		
		System.out.println("Starting " + (MAX_M) + " of "+ MAX_N + " search...");		
		//DEFAULT PARAMETERS, USE DEFAULT IF NOT SET
		//PRINT ARRAY OBJECTS	
		
		//motif_subset(500);
		display_motifs();
		System.out.println();
		add_write_output("");
					
		//used by run()
		if(startPos==0){
			stopPos = motifList.size();
		}
			
		//Creating pareto list with size equal nr of seq
		pareto = new double[nr_of_seq+1];
		
		for(int i=0;i<=nr_of_seq;i++){
			pareto[i]=1;
		}
				
		
		//START COMPUTATION
		for(int i=startPos;i<stopPos;i++){
		
			long starttime = System.currentTimeMillis();
		
			//START DFS FOR EACH ROOT MOTIF
			RealCompositeMotif cm = new RealCompositeMotif();
			cm.expand(i);
			
			if(mode[MACHINE]==CLIENT){
				//UPDATE SERVER
				
				try{
					String command = "update";	
					oos.writeObject(command);
					for(int p=0;p<pareto.length;p++){
						oos.writeObject(new Double(pareto[p]));
					}
					if(ois.readObject().equals("update")){
						for(int p=0;p<pareto.length;p++){
							pareto[p] = ((Double)ois.readObject()).doubleValue();
						}
					}			
				}catch(Exception e){
					e.printStackTrace();
				}	
			}
			
			long elapsed = (System.currentTimeMillis()-starttime)/1000;
			
			//add_output(i + " Time used: " + (elapsed) + "    Time left (approx): " +(motifList.size()*elapsed)/120 + " min");//-starttime/1000));
			System.out.println(i + " Time used: " + (elapsed) + "    Time left (approx): " +(motifList.size()*elapsed)/120 + " min");//-starttime/1000));
		
		}
		
	}
	
	/**
	* Adds debug-strings to list that can be printed
	* @param output string that should be added to the debug output.
	*/
	public static void add_output(String output){
		debugstr.add(output);
	}
	
	/**
	* Adds output for printing to file to an ArrayList
	* @param output string that should be added to the program output.
	* @see OutputWriter
	*/
	public static void add_write_output(String output){
		outputstr.add(output);
	}
	
	/**
	* Returns avg sequence length calculated from current sequence list
	* Used by inputreader.
	* @return average the average length on sequence strings.
	* @see InputReader
	*/
	public static int getAvgSeqLen(){
		int lensum = 0;
		
		for(int i=1;i<sequenceList.size();i+=2){
			lensum+=((String)sequenceList.get(i)).length();
		}
		
		return lensum/(sequenceList.size()/2);
	}
	
	/**
	* Calculates and print a list of counted support added for all single motifs in the Pareto.
	* @return list of support values found.
	*/
   	private static int[] countSupport(){
		//New pareto list based on support
		int[] alt_support = new int[nr_of_seq+1];
					
		for(int i=0; i<pareto.length;i++){
		
			int alt_tot = 0;
			
			if(patterns.containsKey(new Integer(i))){
							
				//Get all patterns
				ArrayList tmpList = (ArrayList)patterns.get(new Integer(i));
								
				//for the bitset-length
				for(int k=0;k<=nr_of_seq;k++){
					
					//support counter
					int alt_supp = 0;
					
					//count total on each position
					for(int j=0;j<tmpList.size();j++){
					
						if(((SingleMotif)tmpList.get(j)).getBitSet().get(k)){
							alt_supp++;
						}
						//System.out.print(((SingleMotif)tmpList.get(j)).getPattern()+ "	");
					}
					alt_tot=alt_tot + alt_supp;
				}//for
			}//if
			alt_support[i] = alt_tot;
		}//for
		
		//Skriv ut den nye lista
		for(int i=0;i<alt_support.length;i++){
		
			if(display.get(PD_COUNTS)){
				System.out.println("CS["+i+ "]: " + Integer.toString(alt_support[i]).replace('.',','));
			}
			if(print.get(PD_COUNTS)){
				add_write_output("CS["+i+"]: " + Integer.toString(alt_support[i]).replace('.',','));
			}
		}
		
		return alt_support;
	}
	
	/**
	* Weights the output from count support to highlight interesting values.
	* @param sList of weighted support vales.
	*/
	private static void weightedSupport(int[] sList){
		double[] wSupport = new double[sList.length];
		
		for(int i=0;i<pareto.length;i++){
			wSupport[i]= Math.pow(sList[i],2.0);			
			if(display.get(PD_WEIGHTS)){
				System.out.println("WS(CS^2)["+i+"]: " + Double.toString(wSupport[i]).replace('.',','));
			}
			if(print.get(PD_WEIGHTS)){
				add_write_output("WS(CS^2)["+i+"]: " + Double.toString(wSupport[i]).replace('.',','));
			}
		}	
	}
	
	/**
	* Binomail significance for the motifs counted support.
	* @param supportList list of support values used in the calculation.
	* @return the probability to observe the support used.
	*/	   	 
	private static int sign_binomial(int[] supportList){
		int I = nr_of_seq;
		long faci = faculty(I);
		
		double bestSum = 0;
		int bestIndex = 0;
		
		for(int x=0;x<=I;x++){
			if(patterns.containsKey(new Integer(x))){
			//System.out.println(x);
			double P = pareto[x];
			
			double sum=0;
			
			for(int n=x;n<=I;n++){
				//System.out.println("kjrer" + Math.pow((1-P),(1-x)));
				sum+=faci/(faculty(I-n)*faculty(n))*Math.pow(P,n)*Math.pow((1-P),(I-n));
			}
			if(sum>=bestSum && sum<1.0){
				bestSum = sum;
				bestIndex = x;
				//System.out.println(sum +" - " +bestIndex);
			}
			if(display.get(PD_BINOMIAL)){
				System.out.println("SB["+x+"]: " + Double.toString(sum).replace('.',','));
			}
			if(print.get(PD_BINOMIAL)){
				add_write_output("SB["+x+"]: " + Double.toString(sum).replace('.',','));
			}
		}
		}
		
		return bestIndex;
	}
	
	/**
	* Calculates the facutly of an int
	* @param mul integer to calculate.
	* @return faculty calculation
	*/
	private static long faculty(int mul){
		long answ = 1;
		for(int i=mul;i>1;i--){
			answ*=i;
		}
		return answ;
		
		/*	
		if(mul==0){
			return 1;
		}else{
			return mul * faculty(mul-1);
		}
		*/
	}
	
	/**
	* Finding likelyhood of observing support over support-total after many trails
	* @param supportList list of support values.
	*/
	private static void sign_total(int[] supportList){
		
		//sum sign over supp in all experiments
		int[] overSupp = new int[nr_of_seq+1];
		
		for(int p=0;p<pareto.length;p++){
			if(patterns.containsKey(new Integer(p))){
				
				ArrayList tmpList = (ArrayList)patterns.get(new Integer(p));
			
				//number of trials
				for(int i = 1;i<=100000;i++){
					//for each sequence
					int poeng = 0;
						
					for(int b=0;b<Mofn.MAX_N;b++){
							
						SingleMotif sm = (SingleMotif)tmpList.get(b);
					
						for(int s=0;s<=nr_of_seq;s++){
							//sample support for given sign (random)
						
							double tall = Math.random();
							
							/*try{
								System.out.println(tall + " vs " + sm.getSign());
								Thread.sleep(500);
							}catch(Exception e){
								System.out.println("Svn avbrutt");
							}
							*/
							
							if(tall<sm.getSign()){
								poeng++;//=sm.getSign();
							}
						}
									
					}//seq
					if(poeng>=supportList[p]){
					//	System.out.println("Wee");
						overSupp[p]++;
					}//if
				}//1000
			}//contains
		}//patterns
		for(int i=0;i<overSupp.length;i++){
			//System.out.println(overSupp[i]);
			if(display.get(PD_EXP)){
				System.out.println("ST["+i+"]: " + Double.toString((double)overSupp[i]/100000).replace('.',','));
			}
			if(print.get(PD_EXP)){
				add_write_output("ST["+i+"]: " + Double.toString((double)overSupp[i]/100000).replace('.',','));
			}
		}
	}

	/** 
	* Applies distance constraints on patterns found after run.
	* @param start start position in sequences
	* @param stop end position in sequences
	*/
	private static void applyDistanceConstraints(int start, int stop){
		//New pareto list based on support
					
		//for each composite motif found
		for(int i=0;i<pareto.length;i++){
			
			//if valid, supp >0
			if(patterns.containsKey(new Integer(i))){
							
				//Get all patterns in CM
				ArrayList tmpList = (ArrayList)patterns.get(new Integer(i));
							
				//for each pattern
				for(int j=0;j<tmpList.size();j++){
					SingleMotif testMotif = ((SingleMotif)tmpList.get(j));
					int[] testData = testMotif.getHitData();
					BitSet testBitSet = testMotif.getBitSet();
					int removed = 0;
					
					//for each hit/pos data
					for(int k=1;k<testData.length;k+=2){
				
						if(testData[k]<start || testData[k]>stop){
							testBitSet.clear(k-1);
							testData[k-1]=-1;
							testData[k]=-1;
							removed++;
						}
						//System.out.print(((SingleMotif)tmpList.get(j)).getPattern()+ "	");
					}
					
					int[] newData = new int[testData.length-removed*2];
					int newIndex = 0;
					for(int k=1;k<testData.length;k++){
						if(testData[k]!=-1){
							newData[newIndex] = testData[k];
							newIndex++;
						}	
					}
					
					testMotif.setHitInfo(testBitSet,newData,newData.length/2,testBitSet.cardinality());
				}//for
			}//if
		}//for
		
	}
	
	/**
	* Calulates significance based up on normal distribution.
	* Finds mean, variance and probability. ONLY WORKS ON SMALL DATASETS.
	* @param meas list of support to check the probability for.
	* @return a list of probabilities found.
	*/
	private static double[] normalDistribution(int[] meas){
		int listSize = 0;
		
		for(int i=0;i<MAX_N;i++){
			listSize+=Math.floor(faculty(motifList.size())/(faculty(motifList.size()-i)*faculty(i)));
		}
		listSize+=MAX_N;
		
		//finding normal distribution for the motifs
		int[] motsupport = new int[listSize];
		int motindex = 0;
		for(int i=0;i<motifList.size();i++){
			exploreSupport(motsupport,motindex,i,0);
		}
		
		Arrays.sort(motsupport);
		
		
		//x p distribution
		int countmeas[]= new int[meas.length];
		for(int a=0;a<meas.length;a++){
			for(int i=0;i<motsupport.length;i++){
				if(motsupport[i]==meas[a]){
					countmeas[a]++;
				}
			}
			countmeas[a]=countmeas[a]/motsupport.length;
		}
		
		//Mean
		double mean = 0.0;
		for(int a=0;a<meas.length;a++){
			mean+= meas[a]*countmeas[a];
		}
		
		//Variance
		double variance = 0.0;
		for(int a=0;a<meas.length;a++){
			variance+=Math.pow(meas[a]-mean,2);
		}
		
		//Probability
		double[] calc_ans = new double[meas.length];
		for(int a=0;a<meas.length;a++){
			for(int b=meas[a];b<nr_of_seq;b++){
				calc_ans[a] += 1/(Math.sqrt(2*Math.PI)*variance)*Math.pow(Math.E,-(1/2)*Math.pow(((meas[a]-mean)/variance),2));
			}
		}
		
		//System.out.println();
		//add_write_output();
		return calc_ans;
	}
	
	/**
	* Used to find the normal distribution of possible support values.
	* @param motsupport empty list, placeholder for values calculated.
	* @param motindex incrementing counter.
	* @param p motif index to explore.
	* @param t number of motifs calculated t<=MAX_M.
	*/
	private static void exploreSupport(int[] motsupport,int motindex, int p, int t){
		p++;
		int partsupp = 0;
		
		while(p<motifList.size() && t<MAX_N){
			SingleMotif sm = (SingleMotif)motifList.get(p);
			partsupp += sm.getSupp();
			t++;
			exploreSupport(motsupport, motindex, p,t);
		}
		
		if(p==MAX_N){
			motsupport[motindex] = partsupp;
			motindex++;
		}
	}
	
	/**
	* This method is used to print or display the list of motifs read in before starting a run.
	* @see InputReader
	*/
	private static void display_motifs(){
		//double sign = 0.0;
		//int counter = 0;
		
		for(int c=0;c<motifList.size();c++){
			SingleMotif mym = (SingleMotif)motifList.get(c);
			
			if(print.get(PD_MOTIFS)){
				add_write_output(c + "   -  Supp: " + mym.getSupp() + "   -  Sign: " + mym.getSign() + "   -  Pattern: "+
					mym.getPattern());
				add_write_output(mym.getBitSet().toString());
			}
			if(display.get(PD_MOTIFS)){		
				System.out.println(c + "   -  Supp: " + mym.getSupp() + "   -  Sign: " + mym.getSign() + "   -  Pattern: "+
					mym.getPattern());
				System.out.println(mym.getBitSet().toString());
			}
			//int[] moreData = mym.getHitData();
			//for(int i=0;i<moreData.length;i++){
				//System.out.println(moreData[i]);
			//}
						
		}
		
	}
	
	/**
	* Creates a subset of the motiflist with size provided.
	* @param size Number of elements in the random subset
	*/
	private static void motif_subset(int size){
		
		while(motifList.size()> size){
			int remove = (int)Math.random()*motifList.size();	
			Object garbage = motifList.remove(remove);
		}
	}
	
	/**
	* Print or displays settings used during a run
	* @param args Arguments used to run program.
	*/
	private static void display_settings(String[] args){
		String param = "";
		for(int i=0;i<args.length;i++){
			param = param + " " + args[i];
		}
		
		System.out.println("Version: " + Mofn.VERSION);
		System.out.println("Parameters: " + param);
		//System.out.println("Motifs: " + motifFile + "  Seqs: " + sequenceFile);
		System.out.println("Start: " + startPos + " - Stop: " + stopPos + " PL: " + pareto.length + " - N: " + (Mofn.MAX_M) + "/" + Mofn.MAX_N);
		System.out.println("Motif count: " + motifList.size());
		if(mode[INPUT]==PVM){
			System.out.println("Threshold: " + threshold + " Percent: " + percent);
		}
		
		add_write_output("Version: " + Mofn.VERSION);
		//add_write_output("Motifs: " + motifFile + "  Seqs: " + sequenceFile);
		add_write_output("Parameters: " + param);
		add_write_output("Start: " + startPos + " - Stop: " + stopPos + " PL: " + pareto.length + " - N: " + (Mofn.MAX_M) + "/" + Mofn.MAX_N);
		add_write_output("Motif count: " + motifList.size());
		if(mode[INPUT]==PVM){
			add_write_output("Threshold: " + threshold + " Percent: " + percent);
		}
	}	
	
	/**
	* This method is used to print or display the Pareto-front list
	*/
	private static void display_pareto(){
		for(int x=0;x<pareto.length;x++){
			double inv_pareto = 1-pareto[x];
			if(display.get(PD_PARETO)){
				System.out.println("Pareto["+x+"]: " + (Double.toString(inv_pareto)).replace('.',','));
			}
			if(print.get(PD_PARETO)){
				add_write_output("Pareto["+x+"]: " + (Double.toString(inv_pareto)).replace('.',','));
			}
			//System.out.print(x + "	" + pareto[x]);
		}
	}
	
	
	/**
	* This method is used to print or display the patterns found
	*/
	private static void display_patterns(){
		for(int i=0; i<pareto.length;i++){
			if(patterns.containsKey(new Integer(i))){
				ArrayList tmpList = (ArrayList)patterns.get(new Integer(i));
				String outStr = "Pattern[" + i+ "]: ";
				
				for(int j=0;j<tmpList.size();j++){
					outStr = outStr + ((SingleMotif)tmpList.get(j)).getPattern()+ "	";
				}
				if(display.get(PD_PATTERNS)){
					System.out.println(outStr);
				}
				if(print.get(PD_PATTERNS)){
					add_write_output(outStr);
				}
			}	
		}
	}
	
	/**
	* Prints or displays the most common single motif from the composite elements.
	*/
	private static void display_mostCommonMotif(){
		TreeMap mcm = new TreeMap();
		
		for(int i=0; i<pareto.length;i++){
			if(patterns.containsKey(new Integer(i))){
				ArrayList tmpList = (ArrayList)patterns.get(new Integer(i));
				//String outStr = Pattern[" + i+ "]: ";
				
				for(int j=0;j<tmpList.size();j++){
					if(mcm.containsKey(((SingleMotif)tmpList.get(j)).getName())){
						int val = ((Integer)mcm.get(((SingleMotif)tmpList.get(j)).getName())).intValue();
						mcm.put(((SingleMotif)tmpList.get(j)).getName(),new Integer(val+1));
					}else{
						mcm.put(((SingleMotif)tmpList.get(j)).getName(),new Integer(1));
					}
				}//for
			}//if	
		}//for
		
		String[] mcm_str = new String[mcm.size()];
		int count = 0;
		
		for(int i=0;i<motifList.size();i++){
			SingleMotif sm = (SingleMotif)motifList.get(i);
			if(mcm.containsKey(sm.getName())){
				
				mcm_str[count] = ((Integer)mcm.get(sm.getName())).toString() + "_" + sm.getName();//System.out.println(sm.getName() + " - " + ((Integer)mcm.get(sm.getName())).intValue());
				count++;
				
				//System.out.println(sm.getName() + " - " + ((Integer)mcm.get(sm.getName())).intValue());
			}
		}
		
		Arrays.sort(mcm_str);
		for(int i=mcm_str.length-1;i>=0;i--){
			StringTokenizer strt = new StringTokenizer(mcm_str[i],"_");
			String cc = strt.nextToken();
			String mm = strt.nextToken();
			
			if(display.get(PD_SINGLEMOTIFS)){
				System.out.println(mm + ": " + cc);
			}
			if(print.get(PD_SINGLEMOTIFS)){
				add_write_output(mm + ": " + cc);
			}
			
		}	
		
	}
	
	/**
	* Prints or displays the list of motif hits combined with sequences
	* @param sequences list of sequences with alternating (name, pattern)
	* @param bestIndex best target pattern found.
	*/
	private static void display_results(Vector sequence, int bestIndex){
		
		//System.out.println("PS: " + patterns.size());
		//System.out.println("BI: "+ bestIndex);
		//System.out.println("SS: " + sequence.size());
		if(display.get(PD_FINAL)){
			System.out.println("Module 1:");// + bestIndex + ":");
		}
		if(print.get(PD_FINAL)){
			add_write_output("Module 1:");//"+bestIndex + ":");
		}
		if(patterns.size()!=0){
			ArrayList bestMotifs = (ArrayList)patterns.get(new Integer(bestIndex));
			/*
			for(int p=0;p<patterns.size();p++){
				if(patterns.containsKey(new Integer(p))){
					System.out.println(p);
				}
			}
			System.out.println("BI:" + bestIndex);
			System.out.println(bestMotifs.size());
			*/
			
			//if(bestMotifs!=null){
			int seqCounter=0;
			for(int s=0;s<sequence.size();s+=2){
				String seqName = (String)sequence.elementAt(s);
				String seqStr = (String)sequence.elementAt(s+1);
				String motNameStr = "";
				
				int firstHit = seqStr.length()+1;
				int lastHit = -1;
				
				for(int m=0;m<bestMotifs.size();m++){
					SingleMotif sm = (SingleMotif)bestMotifs.get(m);
					motNameStr=motNameStr + ", " + sm.getName();
					
					int[] hitData = sm.getHitData();
					
					for(int h=0;h<hitData.length;h+=2){
						//System.out.println(hitData[h]);
						if(hitData[h]==seqCounter){
							if(hitData[h+1]<firstHit){
								firstHit = hitData[h+1];
								
								if(firstHit+sm.getRows()>lastHit){
									lastHit=firstHit+sm.getRows()-1;
									//System.out.println("LH: " + lastHit);
									//System.out.println("SL: " + seqStr.length());
								}
							}
							if(hitData[h+1]>lastHit){
								lastHit = hitData[h+1];
							}
						}//if
					}//for listdata
								
				}//for mot
			//	outputstr= outputstr + ((String)sequence.elementAt(s+1)).substring(0,1);
			//	outputstr= outputstr + sm.getName();
				if(firstHit!=seqStr.length()+1 && lastHit!=-1){
					if(display.get(PD_FINAL)){
						System.out.println(seqName + ", " + firstHit +", " + lastHit +", "+ seqStr.substring(firstHit,lastHit)+motNameStr);
					}
					if(print.get(PD_FINAL)){
						add_write_output(seqName + ", " + firstHit +", " + lastHit +", "+ seqStr.substring(firstHit,lastHit)+motNameStr);
					}
				}else{
					//System.out.println(seqName + "," + firstHit +"," + lastHit +","+ seqStr.substring(firstHit,lastHit)+","+motNameStr);
				}
				seqCounter++;
			}//for seq
		}else{
			if(display.get(PD_FINAL)){
				System.out.println("Empty...");
			}
			if(print.get(PD_FINAL)){
				add_write_output("Empty...");
			}
		}//if null
	}
	
	
	/**
	* Find last non-zero element in a list, can be used to cut of a long empty list.
	* @Param pList list to find the last element in.
	* @returns Last elements index.
	*/
	public static int findLastElement(double[] pList){
		int i;
		for(i=pList.length-1;i>=0;i--){
			if(pList[i]==0){
			}
		}
		return i;
	}
	
	/**
	* Find last non-zero element in a list, can be used to cut of a long empty list.
	* @Param pList list to find the last element in
	* @returns Last elements index
	*/
	public static int findLastElement(int[] pList){
		int i;
		for(i=pList.length-1;i>=0;i--){
			if(pList[i]==0){
			}
		}
		return i;
	}
	
	
	/**
	* Finds scores for PWMs for every position in every sequence. 
	* @param sequences a list of alternating sequence name and pattern.
	* @return A double[] with scores found for each sequence
	*/
	private static ArrayList findMatrixScores(Vector sequences){
		ArrayList allSums = new ArrayList();
		
		for(int m=0;m<motifList.size();m++){
			SingleMotif sm = (SingleMotif)motifList.get(m);
			double[][] mat = sm.getMatrix();
			double[] rowsum = sm.getRowSum();
			
			for(int i=1;i<sequences.size();i+=2){
				//System.out.println(i);
				
				String seqstr = (String)sequences.elementAt(i);
				
				double partSums[] = new double[seqstr.length()];
				
				for(int p=0;p<seqstr.length()-sm.getRows();p++){
					double partSum = 1.0;
					
					for(int c=0;c<sm.getRows();c++){
						partSum*=mat[c][getIndex(seqstr.charAt(p+c))]/rowsum[c];		
					}
					partSums[p] = partSum;
					
				}//single seq
			
				allSums.add(partSums);
			}//tot seqs
		}//motif
		return allSums;
	}//function
	
	/**
	* Translate ACGT letters to indexs 0,1,2,3
	* @return int index
	*/
	private static int getIndex(char c){
		int index = 0;
		c = Character.toUpperCase(c);
		Character ch = new Character(c);
				
		if(ch.equals(new Character('A'))){
			index= 0;
		}else if(ch.equals(new Character('C'))){
			index= 1;
		}else if(ch.equals(new Character('G'))){
			index= 2;
		}else if(ch.equals(new Character('T'))){
			index= 3;
		}
		
		return index;
	}
	
	/**
	* Finds a threshold value for PWMs based on percent included
	* @param allsums Contain scores as double[] for every position in every sequence
	* @return Threshold to include the amount decided on as cutoff-value.
	*/
	private static double findMatrixThreshold(ArrayList allsums){
			int extra = 10;

			double topScores[] = new double[allsums.size()*extra];
			
			for(int s=0;s<allsums.size();s++){
				double tmpScore[] = (double[])allsums.get(s);
				double score[] = new double[tmpScore.length];
				
				for(int t=0;t<tmpScore.length;t++){
					score[t]=tmpScore[t];
				}
				
				Arrays.sort(score);
				int bound = extra;
				if(extra>score.length){
					bound=score.length;
				}
				for(int t=0;t<bound;t++){
					topScores[s*extra+t]=score[score.length-1];
				}
				/*for(int i=0;i<score.length;i++){
					if(score[i]!=0.0){
						System.out.println(score[i]);
					}
				}*/
				
			}
			
			
			
			Arrays.sort(topScores);
			
			/*for(int i=topScores.length-1;i>=0;i--){
				//if(topScores[i]!=0.0){
				System.out.println(topScores[i]);
				//}
			}*/
			
			int thresholdIndex = (int)topScores.length*percent/100;
			//System.out.println("Thre: " + topScores[topScores.length-thresholdIndex]);
			return topScores[topScores.length-thresholdIndex];
			//System.out		
	}
	
	/**
	* Finds PWM support for all the sequences based on threshold
	* @param allScores A list for every hit in every sequence
	* @param thres used during cut-off.
	*/
	private static void findMatrixSupport(ArrayList allScores, double thres){
		
		    //System.out.println("nrofseq:" + nr_of_seq);
		    //System.out.println("seq: " + allScores.size() + " mot: " + motifList.size());
			//double seqScores[] = new double[allScores.size()];
			for(int m=0;m<motifList.size();m++){
				SingleMotif sm = (SingleMotif)motifList.get(m);
		
			    Vector hitInfo = new Vector(); //pos/seq
				
				for(int s=m*(nr_of_seq);s<(m*(nr_of_seq)+(nr_of_seq));s++){
					double seqScore[] = (double[])allScores.get(s);
					
					for(int i=0;i<seqScore.length;i++){
						if(seqScore[i]>=thres){
							//System.out.println(s-(m*(nr_of_seq)));
							hitInfo.add(new Integer(s-(m*(nr_of_seq))));
							hitInfo.add(new Integer(i));
						}
					}//Seq
				}//allseq
				
				BitSet bs = new BitSet(nr_of_seq+1);
				int hitData[] = new int[hitInfo.size()];
			    int htot = hitInfo.size()/2;
			    int hseq = 0;
			    
			    int lastSeqNr = -1;
			    
				for(int j=0;j<hitInfo.size();j+=2){
									
					bs.set(((Integer)hitInfo.elementAt(j)).intValue());
					hitData[j]=((Integer)hitInfo.elementAt(j)).intValue();
					hitData[j+1]=((Integer)hitInfo.elementAt(j+1)).intValue();
					if(((Integer)hitInfo.elementAt(j)).intValue()!=lastSeqNr){
						hseq++;
						lastSeqNr = ((Integer)hitInfo.elementAt(j)).intValue();
					}
					
				}
				
				sm.setHitInfo(bs,hitData,hseq,htot); 
			}//mot			
			
	}
	
	/**
	* Finds PWM significance based on random sequences or real background
	* @param thres Value useded to cut-off matches to find significance
	* @param backseqs List of background sequences (optional) null/!null
	*/
	private static void findMatrixSign(double thres, Vector backseqs){
		Vector seqs;
		
		if(backseqs==null){
			InputReader ir = new InputReader();
			seqs = ir.createRandomSequences(1000,1000);
		}else{
			seqs = backseqs;
		}
		
		for(int m=0;m<motifList.size();m++){
			SingleMotif sm = (SingleMotif)motifList.get(m);
			double[][] mat = sm.getMatrix();
			double[] rowsum = sm.getRowSum();
			int countHits = 0;
			double totSum = 0.0;
			
			for(int i=1;i<seqs.size();i+=2){
				//System.out.println(i);
				
				String seqstr = (String)seqs.elementAt(i);
				
				for(int p=0;p<seqstr.length()-sm.getRows();p++){
					double partSum = 1.0;
					
					for(int c=0;c<sm.getRows();c++){
						partSum*=mat[c][getIndex(seqstr.charAt(p+c))]/rowsum[c];
						
					}
					
					if(partSum>=thres){
						countHits++;
						break;
					}
					
				}//single seq
				
				//totSum += (double)countHits/((double)seqstr.length()-(double)sm.getRows());
				//countHits = 0;
			}//tot seqs
			
						
			//double signFound = (double)countHits/((double)(seqs.size()/2*1000));
			double signFound = (double)countHits/((double)(seqs.size()/2));
			sm.setSign(signFound);
			
		}//motif
		
		sortMotifList();
		//motifList = sortArrayList();
	}
	
	
	/**
	* Sorts the arraylist of motifs after new significance values are applied.
	*/
	private static void sortMotifList(){
		//SORT LIST AFTER DESCENDING SIGNIFICANCE FOUND		
		ArrayList newMotifList = new ArrayList();
		
		for(int i=0;i<motifList.size();i++){
			SingleMotif sortMotif = (SingleMotif)motifList.remove(0);
			
			if(newMotifList.size()==0){
				//System.out.println(mSign); 
				newMotifList.add(sortMotif);
			}else{
				//ACENDING
				for(int s=0;s<newMotifList.size();s++){
					SingleMotif inListMotif = (SingleMotif)newMotifList.get(s);
					
					if(sortMotif.getSign()<=inListMotif.getSign()){
						newMotifList.add(s,sortMotif);
						sortMotif=null;
						break;
					}
				}
				if(sortMotif!=null){
					newMotifList.add(sortMotif);
					sortMotif=null;
				}		
				
				/*
				//DESCENDING
				int ms = 0;
					
				do{
					SingleMotif inListMotif = (SingleMotif)newMotifList.get(ms);
					
					if (ms>=newMotifList.size()-1){
						newMotifList.add(sortMotif);
						sortMotif=null;
					}else if(sortMotif.getSign()>=inListMotif.getSign()){
						newMotifList.add(ms,sortMotif);
						sortMotif=null;
					}
					ms++;	
				}while (sortMotif!=null);//(ms<= listOfMotifs.size() && 
				*/
						
			}//if
		}
		motifList = newMotifList;
	}
	
	/**
	* Finds motif significance (ACGT/IUPAC) based on background
	* @param backseq List of sequences with alternating (name, pattern)
	*/
	private static void findMotifSign(Vector backseq){
	
		for(int t=0;t<motifList.size();t++){
			SingleMotif sm = (SingleMotif)motifList.get(t);
			int hits = 0;

			//PATTERN TO CHECK
			//String motstr = mot.getPattern();
			//TRANSLATE IUPAC INTO ACID...
			String motstr = sm.getPattern();
			motstr = motstr.toUpperCase();
			motstr = translateIUPAC(motstr);
			
			
			//FIND ACTUAL PATTERN LENGTH
			int motlen = 0;
			boolean between = false;
			int betweenloop = 0;
			
			for(int l=0;l<motstr.length();l++){
				if(between){
					if(motstr.substring(l,l+1).equals("]")){
						if(betweenloop==1){
							between = false;
						}
						betweenloop--;
					}else if(motstr.substring(l,l+1).equals("[")){
						betweenloop++;
					}else{
					}
					
				}else{
					if(motstr.substring(l,l+1).equals("[")){
						if(betweenloop==0){
							motlen++;
							between=true;
						}
						betweenloop++;
					}else{
						motlen++;
					}
				}
			}
			
			//System.out.println("Motlen: " + motlen);
				
			//FOR EACH BACKGROUND SEQ
			for(int i=1;i<backseq.size();i+=2){
				String seqstr = (String)backseq.get(i);
				seqstr = seqstr.toUpperCase();
				
				for(int p=0;p<seqstr.length()-motlen;p++){
					//System.out.println("P: " + p);
				
					boolean mismatch = false;
					boolean inside = false;
					int insideloop = 0;
					
					boolean foundletter = false;
					int rp = 0;
					
					//while(m<motlen && !mismatch){ 
					for(int m=0;m<motstr.length();m++){
						
						if(inside){
							if(motstr.substring(m,m+1).equals("[")){
								insideloop++;
								
							}else if(motstr.substring(m,m+1).equals("]")){
								
								if(insideloop==1){
									inside=false;
									if(!foundletter){
										mismatch=true;
										break;
									}
									insideloop--;
									rp++;
								}
							}else if(motstr.substring(m,m+1).equals(seqstr.substring(p+rp,p+rp+1))){
								foundletter=true;
							}
						}else if(motstr.substring(m,m+1).equals("[")){
							inside=true;
							foundletter = false;
							insideloop++;
						}else if(motstr.substring(m,m+1).equals(seqstr.substring(p+rp,p+rp+1))){
							rp++;
						}else{
							rp++;
							mismatch = true;
							break;
						}
	
					}//for mot
					if(!mismatch){
						hits++;
						break;
					}
				}//seqpos
			}//for seg
			sm.setSign((double)hits/(double)backseq.size()/2);
		}//for mot
		
		sortMotifList();
	}//function
	
	/**
	* Translate IUPAC characters into patterns with []
	* @param transstr Pattern to translate.
	* @return The translated pattern.
	*/
	private static String translateIUPAC(String transstr){
		transstr.toUpperCase();
		transstr = transstr.replaceAll("R","[AG]");
		transstr = transstr.replaceAll("Y","[CTU]");
		transstr = transstr.replaceAll("K","[GTU]");
		transstr = transstr.replaceAll("M","[AC]");
		
		transstr = transstr.replaceAll("S","[GC]");
		transstr = transstr.replaceAll("W","[ATU]");
		transstr = transstr.replaceAll("B","[GTCU]");
		transstr = transstr.replaceAll("D","[GATU]");
		transstr = transstr.replaceAll("H","[ACTU]");
		transstr = transstr.replaceAll("V","[GCA]");
		transstr = transstr.replaceAll("N","[ACGTU]");
		transstr = transstr.replaceAll("T","[TU]");
	
		return transstr;
	}
	
			
	/**
	* Starts the server mode for a given number of clients
	* @param nrClients The number of Clients that will connect to the server.
	* @throws Exception If IO error occurs.
	*/
	private static void startSerializedServer(int nrClients) throws Exception{
		//while running
		ArrayList sockets = new ArrayList();
		
		//CALCULATE SUBWORK
		int subWork = (int)Math.ceil(motifList.size()/nrClients);
		System.out.println("Ready for " + (MAX_M) + " of "+ MAX_N + " search...");
		System.out.println("Waiting for clients...");
		
		for(int c=0;c<nrClients;c++){
			Client myClient;
			if(c<(nrClients-1)){
				myClient = new Client(portNr,c,c*subWork,c*subWork-1+subWork);
			}else{
				myClient = new Client(portNr,c,c*subWork,motifList.size());
			}
			System.out.println(((InetAddress)myClient.getIP()).getHostAddress());
			System.out.println("Can handle client: " + c + "  Port: " + (portNr+c));
			
			sockets.add(myClient);
		}
	
		int runningClients = nrClients;
		
		while(runningClients>0){
			for(int c=0;c<nrClients;c++){
				Client client = (Client)sockets.get(c);
				
				while(client.isRunning()){
					try{
						ObjectInputStream ois = client.getOIS();
						ObjectOutputStream oos = client.getOOS();
						
						String command = (String)ois.readObject();
						
						if(command.equals("update")){
							for(int p=0;p<pareto.length;p++){
								pareto[p]=((Double)ois.readObject()).doubleValue();
							}	
							pareto = updatePareto(pareto);
							oos.writeObject("update");
							for(int p=0;p<pareto.length;p++){
								oos.writeObject(new Double(pareto[p]));
							}
							break;
							
						}else if(command.equals("finish")){
							System.out.println(command);
							
							double[] clientPareto = new double[pareto.length];
							for(int p=0;p<pareto.length;p++){
								clientPareto[p]=((Double)ois.readObject()).doubleValue();
							}	
							clientPareto = updatePareto(clientPareto);
							ArrayList smList = new ArrayList();
							
							int nrpatterns = ((Integer)ois.readObject()).intValue();
								
							int p=0;
							while(p<nrpatterns){
		
									int psupp=p;
									
									try{
										int nrsm = ((Integer)ois.readObject()).intValue();
										psupp = ((Integer)ois.readObject()).intValue();
										
										int h=0;
										while(h<nrsm){
											try{
												double hsign= ((Double)ois.readObject()).doubleValue();
												int hsupp = ((Integer)ois.readObject()).intValue();
												String hpattern= (String)ois.readObject();
												int hsize =((Integer)ois.readObject()).intValue();
												int[] hlist = new int[hsize];
													
												int u=0;
												while(u<hsize){
													try{
														hlist[u]= ((Integer)ois.readObject()).intValue();
														u++;
													}catch(Exception e){
													}
												}
												BitSet hbs = new BitSet();
												for(int d=0;d<hlist.length;d+=2){
													hbs.set(hlist[d]);
												}
												SingleMotif addsm = new SingleMotif(0,hsupp,hsign,hpattern,hlist,hbs); 
												if(addsm.getName().equals("MotifX")){
													addsm.setName(hpattern);
												}
												smList.add(addsm);	
												
												h++;
											}catch(Exception e){
												e.printStackTrace();
												break;
											}
										}	
										if(clientPareto[p]>=pareto[p]){
											patterns.put(new Integer(psupp),smList);
										}else{
											smList.clear();
										}
										p++;
									}catch(Exception e){
										e.printStackTrace();
										break;
									}
								//}
							}
							//ArrayList patterns = (ArrayList)ois.readObject();
							client.stopClient();
							runningClients--;
							break;
							
						}else if(command.equals("range")){
							System.out.println("range");
							oos.writeObject(new Integer(client.start));
							oos.writeObject(new Integer(client.stop));
						}else if(command.equals("mofn")){
							System.out.println("range");
							oos.writeObject(new Integer(MAX_M));
							oos.writeObject(new Integer(MAX_N));
						}else{
							break;
						}
					}catch(Exception e){
						e.printStackTrace();
					}
				}//while client
			}//for clients
		}//while running
		
		
		//print results
	}
	
	/**
	* Method used by the server when pareto-lists are recieved from clients
	* @param clientPareto Pareto received from clients
	* @return Updated Pareto list
	*/
	private static double[] updatePareto(double[] clientPareto){
		
		for(int i=clientPareto.length-1;i>=0;i--){
			if(clientPareto[i]>pareto[i]){
				for(int u=i;u>=0;u--){
					pareto[u]=clientPareto[i];
				}
			}
		}
	
		return pareto;
	}
	
	/**
	* Starts the client mode and connects to the server provided
	* @param serverIP IP-adress <xxx.xxx.xxx.xxx>
	*/
	private static void startSerializedClient(String serverIP) throws Exception{
		//STARTUP
		System.out.println(InetAddress.getByName(serverIP).getHostAddress());
		System.out.println(portNr);
		
		sock = new Socket(InetAddress.getByName(serverIP),portNr);
		//sock = new Socket("VELTEV",portNr);
	
		oos = new ObjectOutputStream(sock.getOutputStream());
		ois = new  ObjectInputStream(sock.getInputStream());
		
		stopPos=0;
		
		try{
			while(stopPos==0){
				oos.writeObject("range");
				startPos = ((Integer)ois.readObject()).intValue();			
				stopPos = ((Integer)ois.readObject()).intValue();
			}
		}catch(Exception e){
			e.printStackTrace();
		}
				
		String command = "";
		boolean running = true;
		MAX_N=0;
		try{
			while(MAX_N==0){
				oos.writeObject("mofn");
				MAX_M = ((Integer)ois.readObject()).intValue();			
				MAX_N = ((Integer)ois.readObject()).intValue();
			}
			
		}catch(Exception e){
			e.printStackTrace();
		}	
		
		//RUN()
		
		run();	
						
		//FINISH
		//command="update";
		
		
		command="finish";
		oos.writeObject(command);
		
		for(int p=0;p<pareto.length;p++){
			oos.writeObject(new Double(pareto[p]));
		}
		
		//display_pareto();
		//System.out.println(patterns.size());
		
		oos.writeObject(new Integer(patterns.size()));
		
		for(int p=0;p<pareto.length;p++){
			if(patterns.containsKey(new Integer(p))){
				
				ArrayList smarr = (ArrayList)patterns.get(new Integer(p));
				//System.out.println(smarr.size());
				
				oos.writeObject(new Integer(smarr.size()));
				oos.writeObject(new Integer(p));
				
				for(int h=0;h<smarr.size();h++){
					SingleMotif sm = (SingleMotif)smarr.get(h);
					
					oos.writeObject(new Double(sm.getSign()));
					oos.writeObject(new Integer(sm.getSupp()));
					oos.writeObject(sm.getPattern());
					
					int[] hdl = sm.getHitData();
					oos.writeObject(new Integer(hdl.length));
					
					for(int u=0;u<hdl.length;u++){
						oos.writeObject(new Integer(hdl[u]));
					}
					//oos.writeObject(sm.getBitSet());
					
				}
				
			}
		}
		sock.close();
		System.out.println("Client done...");			
	}
	
}