import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.BitSet;
import java.io.*;

import java.math.*;
import java.util.HashMap;

/**
 * This class handles the output to files.
 * The calculations are done in Mofn and passed to functions here.
 * @author Vetle Valebjrg
 * @version 1.0
 * @see Mofn, InputReader
 */
public class OutputWriter{
	
	private static String nextCounterFile = "nr.txt"; //contains next output nr
	private static String outputFile = "output"; //output filename
	private static String outputDir ="output//"; //output directory
	
	/**
	* Default constructor
	*/
	public OutputWriter(){
	}
	
	/**
	* Constructor with output file as parameter
	* @param newout Filename
	*/
	public OutputWriter(String newout){
		outputFile = newout;
	}
	
	/**
	* Reads, increments and writes new output file number.
	*/
	private void renewOutputNr(){
		//read number
		InputReader ir = new InputReader();
		String nrStr = ir.readNextOutputFileNr(nextCounterFile);
		
		outputFile = outputFile + nrStr + ".txt";
		
		//write new number
		int newnr = Integer.parseInt(nrStr)+1;
		writeNextOutputFileNr(String.valueOf(newnr));
	}
	
	/**
	* This method writes the next output file number to file
	* @exception IOException
	* @see IOException
	*/
	public void writeNextOutputFileNr(String wStr){

		try{
			
			FileWriter myFW = new FileWriter(nextCounterFile);
			BufferedWriter bw = new BufferedWriter(myFW);
			
			myFW.write(wStr);
			myFW.close();
			
		}catch(Exception e){
				String errorstr = "writeNextOutputFileNr";
				System.err.println("Error during " + errorstr);
		}
	}//function
	
	/**
	* Prints information from the run to file
	* @param strings List of strings to write to file.
	*/
	public void print_output(ArrayList strings){
		//PRINTING OUTPUT
		//System.out.println(outputFile);
		
		try{
			renewOutputNr();
			BufferedWriter myBW = new BufferedWriter(new FileWriter(outputDir + outputFile));

								
			for(int x=0;x<strings.size();x++){
				myBW.write((String)strings.get(x));
				myBW.newLine();
			}
	
			myBW.close();
		}catch(Exception e){
			e.printStackTrace();
			//System.out.println("Output file error");
		}
	}
	
	
	/**
	* Returns current output file
	* @return Filename used.
	*/
	public String getFile(){
		return outputFile;
	}
	
}//class