import java.util.ArrayList;
import java.util.BitSet;

/**
 * This class represents real composite motifs during a run
 * @author Vetle Valebjrg
 * @version 1.0
 * @see CompositeMotif, FictiveCompositeMotif, MotifVariant, SingleMotif
 */
public class RealCompositeMotif extends CompositeMotif implements Cloneable{

	
	/**
	* Default constructor used by subclass Motifs
	* @see Motif
	*/
	public RealCompositeMotif(){
		super();
		
		MotifVariant firstMV = new MotifVariant();
		firstMV.makeFirst();
		motVarHolder.put(makeKey(0,0),firstMV);
		
		MotifVariant dummyMV = new MotifVariant();
		dummyMV.makeDummy();
		motVarHolder.put(makeKey(1,0),dummyMV);
	}
	
	/**
	* Constructor used during cloning of composite elements
	*/
	public RealCompositeMotif(CompositeMotif cm){
	 	super(cm);
	}
	 
	
	/**
	* Method used to explore the DFS from Mofn
	* @param n Current position in motif list as int.
	* @see Mofn
	*/
	public void expand(int n){
		
		SingleMotif latestMotif = (SingleMotif)Mofn.motifList.get(n);
		singleMotifHolder.add(latestMotif);
		
		int nt = singleMotifHolder.size();
		int t = calc(nt,latestMotif,"expanding");
		
		if(Mofn.PRUNE){
			while(n<Mofn.motifList.size()-1 && nt<Mofn.MAX_N){
							
				n++;
				//PRUNE, CHECK TO SEE IF SEARCH CAN BE STOPPED
				FictiveCompositeMotif fcm = new FictiveCompositeMotif((CompositeMotif)this.clone());
				fcm.setName("prune");
				MotifVariant chkMV = (MotifVariant)motVarHolder.get(makeKey(t,nt)); 
					
				if(fcm.prune(n,latestMotif.getSign(),Mofn.MAX_N-nt)<Mofn.pareto[chkMV.getSupport()]){
					
					RealCompositeMotif rcm = new RealCompositeMotif((CompositeMotif)this.clone());
					rcm.setName("expand");
					rcm.expand(n);
				}else{
					//System.out.println("pruned");
					continue;
					
				}
			}
		}else{
			while(n<Mofn.motifList.size()-1 && nt<Mofn.MAX_N){
							
				n++;
				RealCompositeMotif rcm = new RealCompositeMotif((CompositeMotif)this.clone());
				rcm.setName("expand");
				rcm.expand(n);
			}
		}		
	
		//CHECK RESULT,UPDATE AND FINISH
		if(singleMotifHolder.size()==Mofn.MAX_N){
			MotifVariant chkMV = (MotifVariant)motVarHolder.get(makeKey(Mofn.MAX_M,Mofn.MAX_N));
			
			//Mofn.add_output(Integer.toString(chkMV.getSupport()));
			//Mofn.add_output(Double.toString(chkMV.getSign()));
			
			if(Mofn.pareto[chkMV.getSupport()]>chkMV.getSign()){
				Mofn.pareto[chkMV.getSupport()]=chkMV.getSign();

				Mofn.patterns.put(new Integer(chkMV.getSupport()),this.singleMotifHolder.clone());
					
				for(int i=chkMV.getSupport();i>=0;i--){
					if(chkMV.getSign()<Mofn.pareto[i]){
						Mofn.pareto[i] = chkMV.getSign();
						
					}//if
				}//for
			}//if
		}//if
	}//function
		

	
	/**
	* Method added to override object clone
	* @return Copy of this module
	* @see Object
	*/
	public Object clone(){

		try {
	  		return super.clone(); 
    	}catch (Exception e) { // Dire trouble!!!
         	throw new InternalError("But we are Cloneable!!!");
      	}
	
	}
	
}