#!/usr/bin/env python
import os,sys
from sets import Set
import ComputeScores as c
from GraphGen import simplePlot

SIMPLE = 0
SPLINE = 1

def Bulk_Score(runfile, outputfile):
    f=open(runfile, 'r')
    fout=open(outputfile, 'w')
    instances = f.readlines()
    f.close()
    fout.write('\t'.join(['evalfile','answerfile','matrixfile','sequencefile','nTP','nFN','nFP','nTN', 
    'nSn','nPPV','nSP','nPC','nCC','nASP','mTP','mFN','mFP','mTN','mSn','mPPV','mSP','mPC','mCC','mASP']) + os.linesep)
    
    # Create empty lists
    nplots,nSn,nPPV,nSP,nPC,nCC,nASP,mplots,mSn,mPPV,mSP,mPC,mCC,mASP = [ [] for i in range(14)]
    
    for instance in instances:
        x = instance.split()
        evalfname = x[0]
        answerfname = x[1]
        matrixfname = x[2]
        seqfname = x[3]
        
        if os.path.isfile(answerfname):
            evalstr = open(evalfname, 'r').readlines()[1:]
            answerstr = open(answerfname, 'r').readlines()[1:]
            matstr = open(matrixfname, 'r').readlines()
        else: 
            continue
            
        motifs = c.get_motifs_matfile(matstr)        # Get all motifs from Matrix file
        eval = c.get_motifsinfo_input(evalstr)      # Return 0 if empty
        if eval==0:                                              # If 0, Continue to next file
            continue
        answer = c.get_motifsinfo_input(answerstr) # Extract information from answer file
        seq = c.read_fasta(seqfname)                     # Get every sequence from FASTA file
        allgenes = list(eval[0].union(answer[0]))     # Find union of genes from evaluation and answer files
        computePosmeasures = eval[3] * answer[3]    # Should we analyse at position level? True/False
        computeMotmeasures = eval[4] * answer[4]    # Should we analyse at motif level? True/False
        evalpos = eval[2]
        answerpos = answer[2]
        evalmotifs = eval[1]
        answermotifs = answer[1]
    
        # Find TP,FP,FN,TN at nucleotide level positions
        numbers_n = c.n_findhits(computePosmeasures,seq.keys(),evalpos,answerpos,seq)
        c.check_sum(numbers_n)
        
        # Find TP,FP,FN,TN at motif level
        numbers_m = c.m_findhits(computeMotmeasures,motifs,seq,evalmotifs,answermotifs)
        c.check_sum(numbers_m)
        
        # Calculate CC,SP,Sn,PPV,PC,ASP
        scores_n = c.calc_scores(numbers_n)
        scores_m = c.calc_scores(numbers_m)
        
        # Write to outfile. Tab separated
        c.write2file(evalfname,answerfname,matrixfname,seqfname,numbers_n,numbers_m,scores_n,scores_m,fout)
        
        # Add scores to lists which is going to be used by simplePlot
        c.add2Lists(scores_n,nCC,nSP,nSn,nPPV,nPC,nASP)
        c.add2Lists(scores_m,mCC,mSP,mSn,mPPV,mPC,mASP)
        
    fout.close()
    
    # Make final Plot data. 
    nplots = c.makePlotData(nplots,nCC,nSP,nSn,nPPV,nPC,nASP,0,SIMPLE)
    mplots = c.makePlotData(mplots,mCC,mSP,mSn,mPPV,mPC,mASP,1,SIMPLE)
    
    # Plot
    simplePlot(nplots,showWindow=True, fname=sys.argv[2][:-4]+"n",xlabel = 'Dataset',ylabel='Value',title='TransCompel at nucleotide level',axis=[1,len(nCC),-1,1.1])
    simplePlot(mplots,showWindow=True, fname=sys.argv[2][:-4]+"m",xlabel = 'Dataset',ylabel='Value',title='TransCompel at motiflevel',axis=[1,len(mSP),-1,1.1])

Bulk_Score(sys.argv[1], sys.argv[2])
