#!/usr/bin/env python
# 
# Fixed: Does not support semicolon between motifs
# Fixed: Does not support sequences without true hits (Seqs in input that are not in answer or predictions are not considered. Should have been TN.)

import os,sys
from sets import Set

def read_fasta(filename):
    cur_key = ""
    seqs = {}
    for line in open(filename):
        if line[0] == ">":
            cur_key = line[1:].strip()
            seqs[cur_key] = ""
        else:
            seqs[cur_key] += line.strip()
    return seqs

def get_motifs_matfile(matstr):
    motifs = []
    for line in matstr:
        if line.find(">")==0:
            motifs.extend(line[1:].split())
    return motifs
    
def get_motifsinfo_input(inputfile):
    geneset =[]
    positions ={}
    motifs ={}
    compute1=True
    compute2=True
    for line in inputfile:
        if line.find("Empty")==0:
            return 0 
        else:
            gname = line.split(",")[0]
            geneset.append(gname)
            startpos = line.split(",")[1]
            endpos = line.split(",")[2]
            motiflist = line.split(",")[4]
            if len(motiflist)==1:
                motiflist = motiflist[0].split(';')
            if positions.get(gname)==None:
                if startpos.find('*')>=0 or endpos.find('*')>=0:
                    compute1=False
                else:
                    positions[gname] = [int(startpos), int(endpos)]
                if motiflist.find('*')>=0:
                    compute2=False
                else:
                    motifs[gname] = [x.strip() for x in line.split(',')[4:] ]
            else: 
                if startpos.find('*')>=0 or endpos.find('*')>=0:
                    compute1=False
                else:
                    positions[gname].extend([int(startpos), int(endpos)])
                if motiflist.find('*')>=0:
                    compute2=False
                else:
                    motifs[gname].extend([x.strip() for x in line.split(',')[4:]])        
    geneset =Set(geneset)
    return [geneset,motifs,positions,compute1,compute2]

def n_findhits(compute,allgenes,evalpos,answerpos,seq):
    if compute:
        FP=TP=TN=FN=0
        lengthseq=0
        for gene in seq.keys():
            seqpositions = Set([])
            evalpositions = Set([])
            answerpositions = Set([])
            
            if seq.get(gene)!=None:     # Feil med sekvenser som var gitt ut. Sekvenser på over 2^16 (65535) nukleotider ble satt til 2^16. Taes ikke med i evaluering. 
                if len(seq.get(gene))>65530:
                    continue
                else:
                    lengthseq +=len(seq.get(gene))
                    seqpositions = Set(range(len(seq.get(gene))))
            if evalpos.get(gene)  !=None:
                evalpositions = Set(range(evalpos.get(gene)[0],int(evalpos.get(gene)[1])+1))
            if answerpos.get(gene) != None:
                answerpositions = Set(range(answerpos[gene][0],answerpos.get(gene)[1]))
                
            TP += len(answerpositions.intersection(evalpositions))
            TN += len(seqpositions-answerpositions-evalpositions)
            FN += len(answerpositions-evalpositions)
            FP += len(evalpositions-answerpositions)
            if len(answerpositions.intersection(evalpositions))+len(seqpositions-answerpositions-evalpositions)+len(answerpositions-evalpositions)+len(evalpositions-answerpositions)!=len(seq.get(gene)):
                print "Something is wrong with: " + gene
                print "length of sequence is: " + str(len(seq.get(gene)))
                print "TP = " + str(len(answerpositions.intersection(evalpositions))) +  " TN = " +  str(len(seqpositions-answerpositions-evalpositions))
                + " FN = " + str(len(answerpositions-evalpositions)) + " PF = " + str(len(evalpositions-answerpositions))
                
        return [TP, FN, FP,TN,lengthseq]  

def m_findhits(compute,motifs,seq,evalmotifs,answermotifs):
    if compute:
        allmotifs = Set(motifs)
        FP=TP=TN=FN=0
        lenmotifs=0
        for gene in seq.keys():
            if len(seq.get(gene))>65530:
                continue
            else:
                lenmotifs +=len(allmotifs)
                evamotifs=Set(evalmotifs.get(gene))
                ansmotifs=Set(answermotifs.get(gene))
                
                TP += len(ansmotifs.intersection(evamotifs))
                TN += len(allmotifs-ansmotifs-evamotifs)
                FN += len(ansmotifs-evamotifs)
                FP += len(evamotifs-ansmotifs)
        
        return [TP,FN,FP,TN,lenmotifs]
    
def calc_scores(numbers): 
    Sn= PPV= SP= PC= CC= ASP = '*'
    TP = numbers[0]
    FN = numbers[1]
    FP = numbers[2]
    TN = numbers[3]
    
    if TP+FN !=0:
        Sn = (1.0*TP)/(TP+FN)
    if TP+FP !=0:
        PPV = (1.0*TP)/(TP+FP)
    if TN+FP !=0:
        SP = (1.0*TN)/(TN+FP)
    if TP+FN+FP !=0:
        PC = (1.0*TP)/(TP+FN+FP)
    if pow((TP+FN)*(TN+FP)*(TP+FP)*(TN+FN), .5) !=0:
        CC = (TP*TN-FN*FP)/pow((TP+FN)*(TN+FP)*(TP+FP)*(TN+FN), .5)
    if Sn !='*' and PPV != '*':
        ASP = (Sn+PPV)/2.0

    res = {'Sn':Sn,'PPV':PPV,'SP':SP,'PC':PC,'CC':CC,'ASP':ASP}
    return res
        
 
def add2Lists(scores,CC,SP,Sn,PPV,PC,ASP):
    CC.append(scores.get('CC'))
    SP.append(scores.get('SP'))
    Sn.append(scores.get('Sn'))
    PPV.append(scores.get('PPV'))
    PC.append(scores.get('PC'))
    ASP.append(scores.get('ASP'))

def makePlotData(plots,CC,SP,Sn,PPV,PC,ASP,level,SIMPLE):
    label = ['n','m']
    plots.append({'y': CC,'x': range(1,len(CC)+1),'type': SIMPLE,'lineproperties': {'label':label[level]+'CC', 'marker':'x', 'linewidth':1, 'color':'b','markerfacecolor':'b'}})
    plots.append({'y':SP,'x': range(1,len(CC)+1),'type': SIMPLE,'lineproperties': {'label':label[level]+'SP', 'marker':'x', 'linewidth':1, 'color':'g','markerfacecolor':'g'}})
    plots.append({'y':Sn,'x': range(1,len(CC)+1),'type': SIMPLE,'lineproperties': {'label':label[level]+'SN', 'marker':'x', 'linewidth':1, 'color':'y','markerfacecolor':'y'}})
    plots.append({'y':PPV,'x': range(1,len(CC)+1),'type': SIMPLE,'lineproperties': {'label':label[level]+'PPV', 'marker':'x', 'linewidth':1, 'color':'r','markerfacecolor':'r'}})
    plots.append({'y':PC,'x': range(1,len(CC)+1),'type': SIMPLE,'lineproperties': {'label':label[level]+'PC', 'marker':'x', 'linewidth':1, 'color':'brown','markerfacecolor':'brown'}})
    plots.append({'y':ASP,'x': range(1,len(CC)+1),'type': SIMPLE,'lineproperties': {'label':label[level]+'ASP', 'marker':'x', 'linewidth':1, 'color':'green','markerfacecolor':'green'}})
    
    return plots
        
def write2file(evalfname,answerfname,matrixfname,seqfname,numbers_n,numbers_m,scores_n,scores_m,fout):
    fout.write('\t'.join([evalfname.split('/')[-1],answerfname.split('/')[-1],matrixfname.split('/')[-1],seqfname.split('/')[-1]]) + '\t' +
            '\t'.join([str(x) for x in [numbers_n[0],numbers_n[1],numbers_n[2],numbers_n[3],scores_n.get('Sn'),scores_n.get('PPV'),
            scores_n.get('SP'),scores_n.get('PC'),scores_n.get('CC'),scores_n.get('ASP'),numbers_m[0],numbers_m[1],numbers_m[2],numbers_m[3],
            scores_m.get('Sn'),scores_m.get('PPV'),scores_m.get('SP'),scores_m.get('PC'),scores_m.get('CC'),scores_m.get('ASP')]]) 
            + os.linesep)

def check_sum(numbers):
    if sum(numbers[0:4]) == numbers[4]:
            print "OK. TP+FP+FN+TN = " + str(sum(numbers[0:4])) 
    else:
        print "Not OK. TP+FP+FN+TN = " + str(sum(numbers[0:4])) +  "  Total should be: " + str(numbers[4])
  
