import pylab as p
from scipy import arange, interpolate

SIMPLE = 0
SPLINE = 1
TEXT = 2

def simplePlot(plots, axis=None, xlabel = None, ylabel = None, title = None , fname = "*", showWindow = False):
    p.clf() #Clears current figure (in case the function is called several times in one run) 
    labels = [] #stores labels to be used in the legend
    lines = [] #stores lines to be used in the legend
    edist = Edist() #stores points that are Eucledian distance-dependent
    
    for simpleplot in plots:
        if not (simpleplot.has_key('x') and simpleplot.has_key('y') and simpleplot.has_key('type')):
            print "warning: plot-dictionary must contain 'x'-, 'y'- and 'type'-keys. Following plot omitted:"
            print simpleplot
            continue
        x = simpleplot['x']
        y = simpleplot['y']
        
        if simpleplot['type'] == SPLINE:
            #Rounded graph using a cubic spline
            tck = interpolate.splrep(x,y,s=0)
            xspline = arange(x[0],x[-1],float(x[1]-x[0])/100)
            yspline = interpolate.splev(xspline,tck,der=0)
            line = p.plot(xspline,yspline)
            
            lp = {}
            if simpleplot.has_key('lineproperties'):
                lp = simpleplot['lineproperties']
            
            for k in lp.keys():
                if k.find('marker') != -1: continue
                elif k == 'label':
                    labels.append(lp['label'])
                    lines.append(line)
                else:
                    p.setp(line,k,lp[k])
            if lp.has_key('marker'):
                m = p.plot(x,y,lp['marker'])
                for k in lp.keys():
                    if k.find('line')!= -1 or k == 'label' or k == 'marker': continue
                    p.setp(m,k,lp[k])
            
        elif simpleplot['type'] == SIMPLE:
            if '*' in y:
                continue
            else:
                line = p.plot(x, y)
                lp = {}
            if simpleplot.has_key('lineproperties'):
                lp = simpleplot['lineproperties']
            for k in lp.keys():
                if k=='label':
                    labels.append(lp['label'])
                    lines.append(line)
                p.setp(line, k, lp[k])
        elif simpleplot['type'] == TEXT:
            edistance = -1
            if simpleplot.has_key('dist'):
                edistance = simpleplot['dist']       
            
            tlfont = {}
            if simpleplot.has_key('font'):
                tlfont = simpleplot['font']
            
            
            tx = [""]*len(x)
            if simpleplot.has_key('text'):
                tx = simpleplot['text']
                
            for i in range(len(x)):
                if not edist.isFarther((x[i], y[i]), edistance): continue
                p.text(x[i], y[i], str(tx[i]), tlfont)
                point = (x[i], y[i])
                edist.addPoint(point)


    if len(lines) > 0:
        leg = p.legend( lines, labels, loc='lower right', shadow=True, pad = 0.5)
        ltext  = leg.get_texts()
        llines = leg.get_lines()
        frame  = leg.get_frame()
        frame.set_height(frame.height*2)
        p.setp(ltext, fontsize='small')
        p.setp(llines, linewidth=1.5)   
    if axis != None: p.axis(axis)
    if xlabel != None: p.xlabel(xlabel)
    if ylabel != None: p.ylabel(ylabel)
    if title != None: p.title(title)
    

    if fname:
        ft = ""
        if title != None: ft = title
        fname = fname.replace('*', ft)				

        if fname:
            p.savefig(fname+".png", dpi=200, facecolor='#cccccc', edgecolor='k')
    if showWindow: p.show()

class Edist:
	def __init__(self, dimensions=[0, 1, 0, 1]):
		self.l = []
		self.d = dimensions

	def isFarther(self, p, mindist):
		
		for q in self.l:
			
			if self.eucledianDistance(p, q) < mindist: return False
		
		return True

	def eucledianDistance(self, pointp, pointq):
		px, py = pointp
		qx, qy = pointq
		return (((px-qx)**2)+((py-qy)**2))**0.5
	
	def addPoint(self, point):
		self.l.append(point)

